/* -*- compile-command: "msdev Stemthing.dsp /make \"Stemthing - Win32 Debug\"" -*-
 */

#include <modeler.h>

#define SPHERE_RADIUS  1
#define SHAPE_CHECK    2
#define STEM_LENGTH    3
#define ROTATE_SPEED   4
#define X_AXIS         5
#define Y_AXIS         6
#define COLOR          7
#define SHININESS      8

MODEL(Stemthing);

int Model_SetupCmd( ClientData clientdata, Tcl_Interp* interp,
		    int objc, Tcl_Obj* const objv[] )
{
    scale( interp, X_AXIS, "x axis", 0.0, 1.0, 0.01, 0.0 );
    scale( interp, Y_AXIS, "y axis", 0.0, 1.0, 0.01, 0.0 );
    scale( interp, ROTATE_SPEED, "rotation speed", 0.0, 360.0, 1.0, 0.0 );
    
    scale( interp, STEM_LENGTH, "stem length", 0.1, 5.0, 0.01, 1.0 );
    scale( interp, SPHERE_RADIUS, "node size", 0.2, 3.0, 0.01, 1.0 );

    scale( interp, SHININESS, "shininess", 0.0, 128.0, 0.1, 100.0 );
    
    checkbox( interp, COLOR, "green?", 0 );
    
    radiobutton( interp, SHAPE_CHECK+10, "shape", 0,
		 "sphere", "cone", "cylinder", "box", NULL );
    menu( interp, SHAPE_CHECK, "shape", 0,
	  "sphere", "cone", "cylinder", "box", NULL );
    
    return TCL_OK;
}

int Model_ResizeCmd( ClientData clientdata, Tcl_Interp* interp,
		     int objc, Tcl_Obj* const objv[] )
{
    int width, height;
    
    if ( objc != 3 )
    {
	Tcl_SetResult( interp, "wrong # args: should be \"width height\"",
		       TCL_STATIC );
	return TCL_ERROR;
    }

    if ( Tcl_GetIntFromObj( interp, objv[1], &width ) != TCL_OK )
	return TCL_ERROR;
    if ( Tcl_GetIntFromObj( interp, objv[2], &height ) != TCL_OK )
	return TCL_ERROR;

    glMatrixMode( GL_PROJECTION );
    glLoadIdentity();
    gluPerspective( 30.0, (double)width / height, 1.0, 100.0 );
    glViewport( 0, 0, width, height );

    glClearColor( 0.0, 0.0, 0.0, 0.0 );

    return TCL_OK;
}

void capped_line( double );

int Model_RedrawCmd( ClientData clientdata, Tcl_Interp* interp,
		     int objc, Tcl_Obj* const objv[] )
{
    double length;

    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    draw_mode_setup();
    
    glMatrixMode( GL_MODELVIEW );
    glLoadIdentity();
    glTranslated( 0.0, 0.0, -15.0 );
    glRotated( get_time() * get_control_d( ROTATE_SPEED ),
	       get_control_d( X_AXIS ),
	       get_control_d( Y_AXIS ),
	       0.0 );
    
    length = get_control_d( STEM_LENGTH );

    shininess( get_control_d( SHININESS ) );
    
    capped_line( length );
    
    glPushMatrix();
    glRotated( 90.0, 1.0, 0.0, 0.0 );
    capped_line( length );
    glPopMatrix();
    
    glPushMatrix();
    glRotated( 90.0, 0.0, 1.0, 0.0 );
    capped_line( length );
    glPopMatrix();
    
    return TCL_OK;
}

void capped_line( double length )
{
    
    glPushMatrix();
    glScaled( 0.4, 0.4, length );
    glTranslated( 0.0, 0.0, -0.5 );
    ambient_color( 0.5, 0.5, 0.5 );
    diffuse_color( 0.5, 0.5, 0.5 );
    cylinder( 1.0, 1.0, 1.0 );
    glPopMatrix();

    glPushMatrix();
    glTranslated( 0.0, 0.0, length/2.0 );
    glScaled( 1.0, 1.0, get_control_d( SPHERE_RADIUS ) );
    if ( get_control_i( COLOR ) )
    {
	ambient_color( 0.1, 0.7, 0.3 );
	diffuse_color( 0.1, 0.7, 0.3 );
	specular_color( 1.0, 0.0, 0.0 );
    }
    else
    {
	ambient_color( 0.1, 0.3, 0.7 );
	diffuse_color( 0.1, 0.3, 0.7 );
    }
    switch( get_control_i( SHAPE_CHECK ) )
    {
	case 0:
	    sphere( 1.0 );
	    break;
	case 1:
	    cylinder( 1.0, 1.0, 0.0 );
	    break;
	case 2:
	    cylinder( 1.0, 1.0, 1.0 );
	    break;
	case 3:
	    box( 1.0, 4.0, 9.0 );
	    break;
    }
    glPopMatrix();
	
    glPushMatrix();
    glRotated( 180.0, 1.0, 0.0, 0.0 );
    glTranslated( 0.0, 0.0, length/2.0 );
    glScaled( 1.0, 1.0, get_control_d( SPHERE_RADIUS ) );
    if ( get_control_i( COLOR ) )
    {
	ambient_color( 0.1, 0.7, 0.3 );
	diffuse_color( 0.1, 0.7, 0.3 );
	specular_color( 1.0, 0.0, 0.0 );
    }
    else
    {
	ambient_color( 0.1, 0.3, 0.7 );
	diffuse_color( 0.1, 0.3, 0.7 );
    }
    switch( get_control_i( SHAPE_CHECK ) )
    {
	case 0:
	    sphere( 1.0 );
	    break;
	case 1:
	    cylinder( 1.0, 1.0, 0.0 );
	    break;
	case 2:
	    cylinder( 1.0, 1.0, 1.0 );
	    break;
	case 3:
	    box( 1.0, 4.0, 9.0 );
	    break;
    }
    glPopMatrix();
}


